#include <LiquidCrystal_I2C.h>
#include <SoftwareSerial.h>

#define TIME_VIBRATION_DELAY       800.0f
#define TIME_SOUND_DELAY           800.0f
#define TIME_LIGHT_DELAY           800.0f
#define SECOND                     1000
#define DETECTION_SECONDS          3 
#define VIBRATION_AVAILABLE        300.0f
#define LIGHT_AVAILABLE            200.0f
#define TONE_BUZZER                100
#define COMMAND_OUTPUT             0
#define MOTION_OUTPUT              0
#define LIGHT_OUTPUT               1
#define SOUND_OUTPUT               2
#define VIBRATION_OUTPUT           3


/* Two pins for Bluetooth Module */
const byte RX = 4;
const byte TX = 5;

/* String for empty a line */
String BLANK_LINE = "                    ";

/* Message received via bluetooth */
String message = "";

/* Pins input/output for sensors */
const int sensorPIR = A0;
const int sensorLight = A1;
const int vibrationPin = A2;
const int sensorSound = 8;
const int relayPin = 7;
const int buzzerPin = 9;

/* State for pir sensor */
int statePIR = LOW;

/* Instance of BT */
SoftwareSerial BTSerial(RX, TX);

/* Instance of LCD */
LiquidCrystal_I2C lcd(0x27, 20, 4);


/* Variables which store time in millis */
volatile float t_vibr = 0;
volatile float t_sound = 0;
volatile float t_light = 0;

/* Flags for check outputs of sensors */
volatile bool flagVibration = false;
volatile bool flagSound = false;
volatile bool flagLight = false;

/* Variable to disable checking pir sensor */
volatile bool disablePIR = true;

/* Variable to check if blink the bulb */
bool blinkBulb = false;

/* Time to blink the bulb */
int timeBlinkBulb = 0;

/* Variables to store values of sensors */
volatile int valuePIR = LOW;
volatile int valueLight = LOW;

/* State buzzer */
int stateBuzzer = LOW;

void setup() {
   
  /* Initialize input pins */
  pinMode(sensorPIR, INPUT);
  pinMode(sensorLight, INPUT);
  pinMode(vibrationPin, INPUT);
  pinMode(sensorSound, INPUT);

  /* Initialize output pins */
  pinMode(relayPin, OUTPUT);
  pinMode(buzzerPin, OUTPUT);

  /* Pass the relay in state OPEN
   *  Mode is Normally Opened (NO)
   */
  digitalWrite(relayPin, HIGH);
 
  /* Initialize communications with phone via BT */
  Serial.begin(9600);
  BTSerial.begin(9600);

  /* Initialize LCD */
  lcd.init();
  lcd.backlight();
}



void loop() {

   /* Detect the motion */
   detectMotion();

   /* Send data from Smartphone TO LCD 
    and from LCD TO Smartphone via BT */ 
   communicationViaBT();

   /* Detect the fenomens */
   detectVibration();
   detectSound();
   detectLight();

}

void detectMotion() {
  if (disablePIR == false) {
   
    valuePIR = digitalRead(sensorPIR);
  
    /* Check if any object is detected */
    if (valuePIR == HIGH && statePIR == LOW) {
  
        lcd.clear();
        lcd.setCursor(0, MOTION_OUTPUT);
        lcd.print("Object detected!");
        
        statePIR = valuePIR;

        /* Turn on the light bulb */
        digitalWrite(relayPin, LOW);

        /* Check if state buzzer is active */
        if (stateBuzzer != LOW) {
          tone(buzzerPin, TONE_BUZZER);
        }
  
    } else if (valuePIR == LOW && statePIR == HIGH) {
      
        lcd.setCursor(0, MOTION_OUTPUT);
        lcd.print("No object detected!");
  
        if (stateBuzzer != LOW) {
          noTone(buzzerPin);
        }
        statePIR = valuePIR;
        digitalWrite(relayPin, HIGH);
        
    }
  }
}

void communicationViaBT() {
  
  if (BTSerial.available()) {
    
      lcd.setCursor(0, COMMAND_OUTPUT);
      lcd.print(BLANK_LINE);
      
      message = BTSerial.readString();

      lcd.setCursor(0, COMMAND_OUTPUT);
      lcd.print(message);

      if (message.substring(0, 10) == "BLINK BULB") {
        blinkBulb = true;
        int seconds = message.substring(11).toInt() * SECOND;
        timeBlinkBulb = seconds;

        lcd.setCursor(0, COMMAND_OUTPUT);
        lcd.print("Blinking bulb!");
 
      } else if (message == "TURN OFF BULB") {

        blinkBulb = false;
        digitalWrite(relayPin, HIGH);

        lcd.setCursor(0, COMMAND_OUTPUT);
        lcd.print("Turning off bulb!");
        BTSerial.println("Turning off bulb!");

      } else if (message == "TURN ON BULB") {

        digitalWrite(relayPin, LOW);
      
        lcd.setCursor(0, COMMAND_OUTPUT);
        lcd.print("Turning on bulb!");
        BTSerial.println("Turning on bulb!");

      } else if (message == "BUZZER ON") {
        
          stateBuzzer = HIGH;
          BTSerial.println("Buzzer ON!");
          
      } else if (message == "BUZZER OFF") {
        
          stateBuzzer = LOW;
          BTSerial.println("Buzzer OFF!");
          
      } else if (message == "DISABLE PIR") {
        
        disablePIR = true;
        BTSerial.println("Disable PIR Sensor!");
        
      } else if (message == "ENABLE PIR") {
        
        disablePIR = false;
        BTSerial.println("Enable PIR Sensor!");
        
      } else {
        
        lcd.setCursor(0, COMMAND_OUTPUT);
        lcd.print("Invalid command!");
        BTSerial.println("Invalid command!");
        
      }
   }

   /* Blinking the bulb */
    if (blinkBulb) {
      digitalWrite(relayPin, LOW);
      delay(timeBlinkBulb);
      digitalWrite(relayPin, HIGH);
      delay(timeBlinkBulb);
    }
}

void detectVibration() {
  
   float valueVibration = analogRead(vibrationPin);
   float t_disableVibr = millis();

   if (valueVibration > VIBRATION_AVAILABLE){
      flagVibration = true;
      t_vibr = millis();
   }

   /* Check if vibration of object stopped */
   if (flagVibration && (t_disableVibr - t_vibr > TIME_VIBRATION_DELAY) &&
      (valueVibration < VIBRATION_AVAILABLE)) {
       
       lcd.setCursor(0, VIBRATION_OUTPUT);
       lcd.print(BLANK_LINE);
       flagVibration = false;
       BTSerial.println("Vibration detected!");
   }

   if (flagVibration) {
      lcd.setCursor(0, VIBRATION_OUTPUT);
      lcd.print("Vibration detected!");
   }

}

void detectSound() {

   int valueSound = digitalRead(sensorSound);
  
   float t_disableSound = millis();

   if (valueSound){
      flagSound = true;
      t_sound = millis();
   }

   /* Check if sound stopped */
   if (flagSound && (t_disableSound - t_sound > TIME_SOUND_DELAY) &&
      (valueSound == 0)) {
        
       lcd.setCursor(0, SOUND_OUTPUT);
       lcd.print(BLANK_LINE);
       flagSound = false;
       BTSerial.println("Sound detected!");
   }

   if (flagSound) {
      lcd.setCursor(0, SOUND_OUTPUT);
      lcd.print("Sound detected!");
   }
}

void detectLight() {
 
   valueLight = analogRead(sensorLight);
  
   float t_disableLight = millis();

   if (valueLight >= LIGHT_AVAILABLE){
      flagLight = true;
      t_light = millis();
   }

   Serial.println(valueLight);
   
   /* Check if light stopped */
   if (flagLight && (t_disableLight - t_light > TIME_LIGHT_DELAY) &&
      (valueLight < LIGHT_AVAILABLE)) {
       lcd.setCursor(0, LIGHT_OUTPUT);
       lcd.print(BLANK_LINE);
       flagLight = false;
       BTSerial.println("Light detected!");
       
   }

   if (flagLight) {
      lcd.setCursor(0, LIGHT_OUTPUT);
      lcd.print("Light detected!");
   }
 
}
